//------------------------ CSPThread.h: -----------------------

//*************************************************************
//*****       Copyright 2002 - 2005 Micronovae Ltd       *****
//*****                All rights reserved                *****
//*************************************************************


#ifndef _CSPTHREAD
#define _CSPTHREAD

#include "CSPConf.h"
#include "CSPLib.h"
#ifdef _CSP_WINDOWS_
#include <process.h>
#endif

#ifdef _CSP_LINUX_
#include <pthread.h>
#endif


//---------------------- CSPThread -------------------------------
#ifdef _CSP_WINDOWS_
class CSPLIB_CLASS_EXPORT CSPThread : public CSPObject
{
public:

	CSPThread();
	virtual ~CSPThread();

	unsigned int GetID() const;
	unsigned long GetHandle() const;
	bool Start( void* pvUserParam = 0 );
	unsigned long Suspend();
	unsigned long Resume();
	bool SetPriority( int iLevel );
	int GetPriority() const;
	bool Kill( unsigned long ulExitCode = 0 );
	bool Wait( unsigned long ulTimeout = WAIT_FOREVER );
	unsigned long GetExitCode() const;

//	static bool CSPThread::WaitForThreads( unsigned long ulCount, ... );


	enum
	{
		ABOVE_NORMAL = THREAD_PRIORITY_ABOVE_NORMAL,
		BELOW_NORMAL = THREAD_PRIORITY_BELOW_NORMAL,
		HIGHEST = THREAD_PRIORITY_HIGHEST,
		IDLE = THREAD_PRIORITY_IDLE,
		LOWEST = THREAD_PRIORITY_LOWEST,
		NORMAL = THREAD_PRIORITY_NORMAL,
		TIME_CRITICAL = THREAD_PRIORITY_TIME_CRITICAL,
		ERROR_RETURN = THREAD_PRIORITY_ERROR_RETURN,
		WAIT_FOREVER = INFINITE
	};

protected:

	void* GetUserParam() const;
	virtual void Run() = 0;
	void SetExitCode( unsigned long ulExitCode );
	void SafeDestruction();
	

private:

	unsigned int m_uiThreadID;
	unsigned long m_ulThread;
	HANDLE m_hStart;
	HANDLE m_hEnd;
	void* m_pvUserParameter;
	mutable int m_iPriorityLevel;
	unsigned long m_ulExitCode;
	bool m_bRunning;
	bool m_bKill;


	CSPThread( const CSPThread& );
	CSPThread& operator = ( const CSPThread& );
	static unsigned int __stdcall ThreadProc( void* pCtx );
};
#endif


#ifdef _CSP_LINUX_
class CSPLIB_CLASS_EXPORT CSPThread : public CSPObject
{
public:

	CSPThread();
	virtual ~CSPThread();

	unsigned int GetID() const;
	bool Start( void* pvUserParam = 0 );
//	unsigned long Suspend();
//	unsigned long Resume();
	bool SetPriority( int iLevel );
	int GetPriority() const;
	bool Kill( unsigned long ulExitCode = 0 );
	bool Wait( unsigned long ulTimeout = WAIT_FOREVER );
	unsigned long GetExitCode() const;

//	static bool CSPThread::WaitForThreads( unsigned long ulCount, ... );


	enum
	{
		ABOVE_NORMAL = 0,
		BELOW_NORMAL = 1,
		HIGHEST = 2,
		IDLE = 3,
		LOWEST = 4,
		NORMAL = 5,
		TIME_CRITICAL = 6,
		ERROR_RETURN = 0XFFFFFFFFUL,
		WAIT_FOREVER = 0XFFFFFFFFUL
	};

protected:

	void* GetUserParam() const;
	virtual void Run() = 0;
	void SetExitCode( unsigned long ulExitCode );
	void SafeDestruction();
	

private:

	pthread_t m_ThreadID;
	void* m_pvUserParameter;
	mutable int m_iPriorityLevel;
	unsigned long m_ulExitCode;
	bool m_bRunning;
	bool m_bKill;
	pthread_cond_t m_CondStart;
	pthread_cond_t m_CondEnd;
	pthread_mutex_t m_MutexStart;
	pthread_mutex_t m_MutexEnd;


	CSPThread( const CSPThread& );
	CSPThread& operator = ( const CSPThread& );
	static void* ThreadProc( void* pCtx );
};
#endif

//---------------------- CSPThreadLockBase ---------------------------

class CSPLIB_CLASS_EXPORT CSPThreadLockBase : public CSPObject
{
public:
	CSPThreadLockBase();
	virtual ~CSPThreadLockBase();

	virtual unsigned long Lock() = 0;
	virtual unsigned long Unlock() = 0;
	virtual bool TryLock() = 0;
	unsigned long GetLockCount() const;

protected:
	unsigned long m_ulLockCount;

private:
	CSPThreadLockBase( CSPThreadLockBase& );
	CSPThreadLockBase& operator = ( CSPThreadLockBase& );
};


//---------------------- CSPThreadLock ---------------------------

class CSPLIB_CLASS_EXPORT CSPThreadLock : public CSPThreadLockBase
{
public:
	CSPThreadLock();
	virtual ~CSPThreadLock();

	unsigned long Lock();
	unsigned long Unlock();
	bool TryLock();

protected:
#ifdef _CSP_WINDOWS_
	CRITICAL_SECTION m_CS;
#endif
#ifdef _CSP_LINUX_
	pthread_mutex_t m_Mutex;
#endif

private:
	CSPThreadLock( CSPThreadLock& );
	CSPThreadLock& operator = ( CSPThreadLock& );
};

typedef CSPThreadLock CSPCriticalSection;



//---------------------- CSPThreadLockHandler ---------------------------

class CSPLIB_CLASS_EXPORT CSPThreadLockHandler : public CSPObject
{
public:
	CSPThreadLockHandler( CSPThreadLockBase& TL );
	virtual ~CSPThreadLockHandler();

	unsigned long Lock();
	unsigned long Unlock();
	bool TryLock();
	unsigned long GetLockCount() const;

protected:
	CSPThreadLockBase& CSPTL;
	unsigned long m_ulCount;
};

#endif
